#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import List

from htf.daqmx.edge_counter_input import EdgeCounterInput


class FrequencyInputInaccurate:
    """
    `FrequencyInputInaccurate` is used to measure a frequency using an
    `EdgeCounterInput`.

    In comparison to ``FrequencyInput`` frequencies below 100 Hz can be
    measured, too.
    """

    def __init__(self, descriptor: str) -> None:
        """
        Args:
            descriptor: The DAQmx device and line descriptor, eg.
                `cDaq1Mod1/ctr0`.
        """
        self.edgeCounter = EdgeCounterInput(descriptor=descriptor, initial_count=0, rising_edge=True, count_up=True)

    def sample(self, period: float = 1.0, samples: int = 1) -> List[float]:
        """
        For `samples` times measure the count of edges within one `period` and
        convert it to Hz.

        Returns an array containing the measured frequencies.

        Args:
            period: the time in seconds between two samples.
            samples: the amount of samples.

        Returns:
            list of float: the list of samples.
        """

        edge_counts = []

        for _ in range(samples):
            self.edgeCounter.reset()
            time.sleep(period)
            edge_count = self.edgeCounter.get()
            edge_counts.append(edge_count)

        frequencies = [1.0 / float(period) * edgeCount for edgeCount in edge_counts]

        return frequencies


if __name__ == "__main__":
    import time

    fi = FrequencyInputInaccurate(descriptor="Dev1/ctr0")

    frequencies = fi.sample(period=1, samples=10)

    print("frequencies:", frequencies)
